@@ -11,6 +11,7 @@ from pdm.cli.options import (
     frozen_lockfile_option,
     install_group,
     lockfile_option,
+    override_option,
     packages_group,
     prerelease_option,
     save_strategy_group,
@@ -36,6 +37,7 @@ class Command(BaseCommand):
         lockfile_option,
         frozen_lockfile_option,
         save_strategy_group,
+        override_option,
         update_strategy_group,
         prerelease_option,
         unconstrained_option,
@@ -103,7 +105,7 @@ class Command(BaseCommand):
     ) -> None:
         """Add packages and install"""
         from pdm.cli.actions import do_lock, do_sync
-        from pdm.cli.utils import check_project_file, populate_requirement_names, save_version_specifiers
+        from pdm.cli.utils import check_project_file, save_version_specifiers
         from pdm.models.requirements import parse_requirement
         from pdm.models.specifiers import get_specifier
         from pdm.utils import normalize_name
@@ -114,7 +116,7 @@ class Command(BaseCommand):
             raise PdmUsageError("Cannot use --no-editable with editable packages given.")
         group = selection.one()
         tracked_names: set[str] = set()
-        requirements: dict[str, Requirement] = {}
+        requirements: list[Requirement] = []
         lock_groups = ["default"] if project.lockfile.empty() else project.lockfile.groups
         if lock_groups is not None and group not in lock_groups:
             if project.enable_write_lockfile:
@@ -133,31 +135,26 @@ class Command(BaseCommand):
                 continue
             if r.is_file_or_url:
                 r.relocate(project.backend)  # type: ignore[attr-defined]
-            r.groups = [group]
             key = r.identify()
             r.prerelease = prerelease
             tracked_names.add(key)
-            requirements[key] = r
+            requirements.append(r)
         if requirements:
             project.core.ui.echo(
                 f"Adding {'[bold]global[/] ' if project.is_global else ''}packages to [primary]{group}[/] "
                 f"{'dev-' if selection.dev else ''}dependencies: "
-                + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements.values())
+                + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements)
             )
+        group_deps = project.add_dependencies(requirements, group, selection.dev or False, write=False)
         all_dependencies = project.all_dependencies
-        group_deps = all_dependencies.setdefault(group, {})
+        all_dependencies[group] = group_deps
         if unconstrained:
             if not requirements:
                 raise PdmUsageError("--unconstrained requires at least one package")
-            for req in group_deps.values():
+            for req in group_deps:
                 req.specifier = get_specifier("")
-        group_deps.update(requirements)
-        reqs = [
-            r for g, deps in all_dependencies.items() if lock_groups is None or g in lock_groups for r in deps.values()
-        ]
-        # pre-write the named dependencies to the pyproject.toml to make them recognized by the resolver
-        named_req = {k: r for k, r in requirements.items() if r.is_named}
-        project.add_dependencies(named_req, group, selection.dev or False, write=False)
+
+        reqs = [r for g, deps in all_dependencies.items() if lock_groups is None or g in lock_groups for r in deps]
         with hooks.skipping("post_lock"):
             resolved = do_lock(
                 project,
@@ -171,19 +168,18 @@ class Command(BaseCommand):
 
         # Update dependency specifiers and lockfile hash.
         deps_to_update = group_deps if unconstrained else requirements
-        save_version_specifiers({group: deps_to_update}, resolved, save)
+        save_version_specifiers(deps_to_update, resolved, save)
         hooks.try_emit("post_lock", resolution=resolved, dry_run=dry_run)
         if not dry_run:
             project.add_dependencies(deps_to_update, group, selection.dev or False)
             project.write_lockfile(project.lockfile._data, False)
-        populate_requirement_names(group_deps)
         if sync:
             do_sync(
                 project,
                 selection=GroupSelection(project, groups=[group], default=False),
                 no_editable=no_editable and tracked_names,
                 no_self=no_self or group != "default",
-                requirements=list(group_deps.values()),
+                requirements=group_deps,
                 dry_run=dry_run,
                 fail_fast=fail_fast,
                 hooks=hooks,
