@@ -8,10 +8,11 @@ from pdm.cli.filters import GroupSelection
 from pdm.cli.hooks import HookManager
 from pdm.cli.options import (
     dry_run_option,
+    frozen_lockfile_option,
     groups_group,
     install_group,
     lockfile_option,
-    no_lock_option,
+    override_option,
     skip_option,
     venv_option,
 )
@@ -25,9 +26,10 @@ class Command(BaseCommand):
         *BaseCommand.arguments,
         groups_group,
         install_group,
+        override_option,
         dry_run_option,
         lockfile_option,
-        no_lock_option,
+        frozen_lockfile_option,
         skip_option,
         venv_option,
     )
@@ -43,12 +45,9 @@ class Command(BaseCommand):
     def install_plugins(self, project: Project) -> None:
         from pdm.environments import PythonEnvironment
         from pdm.installers.core import install_requirements
-        from pdm.models.requirements import parse_requirement
+        from pdm.models.requirements import parse_line
 
-        plugins = [
-            parse_requirement(r[3:], True) if r.startswith("-e ") else parse_requirement(r)
-            for r in project.pyproject.plugins
-        ]
+        plugins = [parse_line(r) for r in project.pyproject.plugins]
         if not plugins:
             return
         plugin_root = project.root / ".pdm-plugins"
