@@ -200,11 +200,13 @@ class Requirement:
         """Return whether the passed in PEP 508 string
         is the same requirement as this one.
         """
-        if line.strip().startswith("-e "):
-            req = parse_requirement(line.split("-e ", 1)[-1], True)
-        else:
-            req = parse_requirement(line, False)
-        return self.key == req.key
+        req = parse_line(line)
+        return (
+            self.key == req.key
+            or isinstance(self, FileRequirement)
+            and isinstance(req, FileRequirement)
+            and self.url == req.url
+        )
 
     @classmethod
     def from_pkg_requirement(cls, req: PackageRequirement) -> Requirement:
@@ -219,7 +221,7 @@ class Requirement:
         if getattr(req, "url", None):
             link = Link(cast(str, req.url))
             klass = VcsRequirement if link.is_vcs else FileRequirement
-            return klass(url=req.url, **kwargs)
+            return klass(url=cast(str, req.url), **kwargs)
         else:
             return NamedRequirement(**kwargs)  # type: ignore[arg-type]
 
@@ -242,6 +244,7 @@ class FileRequirement(Requirement):
     path: Path | None = None
     subdirectory: str | None = None
     check_installable: bool = True
+    _root: Path = dataclasses.field(default_factory=Path.cwd, repr=False)
 
     def __post_init__(self) -> None:
         super().__post_init__()
@@ -293,7 +296,7 @@ class FileRequirement(Requirement):
             return None
         if self.path.is_absolute():
             try:
-                result = self.path.relative_to(Path.cwd()).as_posix()
+                result = self.path.relative_to(self._root).as_posix()
             except ValueError:
                 return self.path.as_posix()
         else:
@@ -330,14 +333,19 @@ class FileRequirement(Requirement):
         if path == ".":
             path = ""
         self.url = backend.relative_path_to_url(path)
+        self._root = backend.root
+
+    @property
+    def absolute_path(self) -> Path | None:
+        return self._root.joinpath(self.path) if self.path else None
 
     @property
     def is_local(self) -> bool:
-        return self.path and self.path.exists() or False
+        return (path := self.absolute_path) is not None and path.exists()
 
     @property
     def is_local_dir(self) -> bool:
-        return self.is_local and cast(Path, self.path).is_dir()
+        return self.is_local and cast(Path, self.absolute_path).is_dir()
 
     def as_file_link(self) -> Link:
         from unearth import Link
@@ -433,12 +441,12 @@ class VcsRequirement(FileRequirement):
 
 def filter_requirements_with_extras(
     requirement_lines: list[str], extras: Sequence[str], include_default: bool = False
-) -> list[str]:
+) -> list[Requirement]:
     """Filter the requirements with extras.
     If extras are given, return those with matching extra markers.
     Otherwise, return those without extra markers.
     """
-    result: list[str] = []
+    result: list[Requirement] = []
     for req in requirement_lines:
         _r = parse_requirement(req)
         req_extras = get_marker("")
@@ -451,7 +459,7 @@ def filter_requirements_with_extras(
         # The requirement has no extras while requested extras are empty or include_default is True, or
         # The requirement has extras, in which case the `evaluate()` test must have been passed.
         if not req_extras.is_any() or include_default or not extras:
-            result.append(_r.as_line())
+            result.append(_r)
 
     return result
 
@@ -467,6 +475,12 @@ def parse_as_pkg_requirement(line: str) -> PackageRequirement:
         return PackageRequirement(new_line)
 
 
+def parse_line(line: str) -> Requirement:
+    if line.startswith("-e "):
+        return parse_requirement(line[3:].strip(), editable=True)
+    return parse_requirement(line)
+
+
 def parse_requirement(line: str, editable: bool = False) -> Requirement:
     m = _vcs_req_re.match(line)
     r: Requirement
